@@ -188,6 +188,20 @@ describe MessagePack::Unpacker do
     objects.should == [sample_object] * 4
   end
 
+  it 'feed and each empty string' do
+    raw = sample_object.to_msgpack.to_s
+    objects = []
+
+    unpacker.feed("")
+    unpacker.feed(raw)
+    unpacker.feed("")
+
+    unpacker.each { |c|
+      objects << c
+    }
+    objects.should == [sample_object]
+  end
+
   it 'feed_each continues internal state' do
     raw = sample_object.to_msgpack.to_s * 4
     objects = []
@@ -481,13 +495,13 @@ describe MessagePack::Unpacker do
       expect(one.keys.sort).to eq([:type, :class, :unpacker].sort)
       expect(one[:type]).to eq(0x01)
       expect(one[:class]).to eq(ValueOne)
-      expect(one[:unpacker]).to eq(:from_msgpack_ext)
+      expect(one[:unpacker]).to be_a(Proc)
 
       two = list[1]
       expect(two.keys.sort).to eq([:type, :class, :unpacker].sort)
       expect(two[:type]).to eq(0x02)
       expect(two[:class]).to eq(ValueTwo)
-      expect(two[:unpacker]).to eq(:from_msgpack_ext)
+      expect(two[:unpacker]).to be_a(Proc)
     end
 
     it 'returns a Array of Hash, which contains nil for class if block unpacker specified' do
@@ -866,4 +880,66 @@ describe MessagePack::Unpacker do
       end
     end
   end
+
+  it "doesn't allow #dup or #clone" do
+    expect(subject).to_not respond_to :dup
+    expect(subject).to_not respond_to :clone
+  end
+
+  it "doesn't crash when marking an uninitialized buffer" do
+    if RUBY_PLATFORM == "java"
+      pending("THe java extension is missing Unpacker#buffer https://github.com/msgpack/msgpack-ruby/issues/315")
+    end
+
+    stress = GC.stress
+    begin
+      GC.stress = true
+
+      MessagePack::Unpacker.new.buffer
+      Object.new
+    ensure
+      GC.stress = stress
+    end
+  end
+
+  if RUBY_PLATFORM != "java"
+    it "doesn't leak when a recursive unpacker raises" do
+      hash_with_indifferent_access = Class.new(Hash)
+      msgpack = MessagePack::Factory.new
+      msgpack.register_type(
+        0x02,
+        hash_with_indifferent_access,
+        packer: ->(value, packer) do
+          packer.write(value.to_h)
+        end,
+        unpacker: ->(unpacker) { raise RuntimeError, "Ooops" },
+        recursive: true
+      )
+
+      packer = msgpack.packer
+      data = [[[[[[[hash_with_indifferent_access.new]]]]]]]
+      payload = msgpack.dump(data)
+
+      unpacker = msgpack.unpacker
+      2.times do
+        unpacker.buffer.clear
+        unpacker.feed(payload)
+        expect {
+          unpacker.full_unpack
+        }.to raise_error(RuntimeError, "Ooops")
+      end
+
+      memsize = ObjectSpace.memsize_of(unpacker)
+
+      10.times do
+        unpacker.buffer.clear
+        unpacker.feed(payload)
+        expect {
+          unpacker.full_unpack
+        }.to raise_error(RuntimeError, "Ooops")
+      end
+
+      expect(memsize).to eq ObjectSpace.memsize_of(unpacker)
+    end
+  end
 end
